package ch.std.jumpstartreactive;

import java.util.ArrayList;
import java.util.List;

import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.WebApplicationType;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import reactor.core.publisher.Flux;

@SpringBootApplication
public class BackpressureApp implements ApplicationRunner {

	Logger logger = LoggerFactory.getLogger(BackpressureApp.class);

	public static void main(String[] args) throws Exception {
		SpringApplication app = new SpringApplication(BackpressureApp.class);
		app.setWebApplicationType(WebApplicationType.NONE);
		app.run(args);
	}

	@Override
	public void run(ApplicationArguments args) {
		this.subscribe1();
		this.subscribe2();
		this.subscribe3();
		this.subscribe4();
	}

	List<Integer> elements = new ArrayList<>();

	private void subscribe1() {
		logger.info("subscribe1 no limit no request start");
		Flux.just(1, 2, 3, 4, 5, 6, 7, 8, 9).log().subscribe(new Subscriber<Integer>() {
			@Override
			public void onSubscribe(Subscription s) {
				logger.info("onSubscribe");
				s.request(Long.MAX_VALUE);
			}

			@Override
			public void onNext(Integer i) {
				logger.info("onNext, i = " + i);
				elements.add(i);
			}

			@Override
			public void onError(Throwable t) {
				logger.info("onError");
			}

			@Override
			public void onComplete() {
				logger.info("onComplete");
			}
		});
		logger.info("subscribe1 end");
	}

	private void subscribe2() {
		logger.info("subscribe2 limit(2) no request start");
		Flux.just(1, 2, 3, 4, 5, 6, 7, 8, 9).log().limitRate(2).subscribe(new Subscriber<Integer>() {
			@Override
			public void onSubscribe(Subscription s) {
				logger.info("onSubscribe");
				s.request(Long.MAX_VALUE);
			}

			@Override
			public void onNext(Integer i) {
				logger.info("onNext, i = " + i);
				elements.add(i);
			}

			@Override
			public void onError(Throwable t) {
				logger.info("onError");
			}

			@Override
			public void onComplete() {
				logger.info("onComplete");
			}
		});
		logger.info("subscribe2 end");
	}

	private void subscribe3() {
		logger.info("subscribe3 no limit request(2) start");
		Flux.just(1, 2, 3, 4, 5, 6, 7, 8, 9).log().subscribe(new Subscriber<Integer>() {
			private Subscription s;
			private int onNextAmount;

			@Override
			public void onSubscribe(Subscription s) {
				logger.info("onSubscribe");
				this.s = s;
				s.request(2);
			}

			@Override
			public void onNext(Integer i) {
				logger.info("onNext, i = " + i);
				elements.add(i);
				onNextAmount++;
				if (onNextAmount % 2 == 0) {
					s.request(2);
				}
			}

			@Override
			public void onError(Throwable t) {
				logger.info("onError");
			}

			@Override
			public void onComplete() {
				logger.info("onComplete");
			}
		});
		logger.info("subscribe3 end");
	}

	private void subscribe4() {
		logger.info("subscribe4 limit(2) request(2) start");
		Flux.just(1, 2, 3, 4, 5, 6, 7, 8, 9).log().subscribe(new Subscriber<Integer>() {
			private Subscription s;
			private int onNextAmount;

			@Override
			public void onSubscribe(Subscription s) {
				logger.info("onSubscribe");
				this.s = s;
				s.request(2);
			}

			@Override
			public void onNext(Integer i) {
				logger.info("onNext, i = " + i);
				elements.add(i);
				onNextAmount++;
				if (onNextAmount % 2 == 0) {
					s.request(2);
				}
			}

			@Override
			public void onError(Throwable t) {
				logger.info("onError");
			}

			@Override
			public void onComplete() {
				logger.info("onComplete");
			}
		});
		logger.info("subscribe4 end");
	}

}
